@@ -13,7 +13,6 @@
 ##############################################################################
 """Test execution
 """
-
 import errno
 import gc
 import io
@@ -30,6 +29,7 @@ import unittest
 import warnings
 from contextlib import contextmanager
 from io import StringIO
+from pathlib import Path
 
 import zope.testrunner
 import zope.testrunner._doctest
@@ -50,6 +50,7 @@ from zope.testrunner import threadsupport
 from zope.testrunner.find import _layer_name_cache
 from zope.testrunner.find import import_name
 from zope.testrunner.find import name_from_layer
+from zope.testrunner.formatter import XMLOutputFormattingWrapper
 from zope.testrunner.layer import EmptyLayer
 from zope.testrunner.layer import EmptySuite
 from zope.testrunner.layer import UnitTests
@@ -201,6 +202,9 @@ class Runner:
                 for feature in self.features:
                     feature.report()
 
+        if self.options.xmlOutput:
+            self.options.output.writeXMLReports()
+
     def configure(self):
         if self.args is None:
             self.args = sys.argv[:]
@@ -225,6 +229,12 @@ class Runner:
         options.resume_layer = resume_layer
         options.resume_number = resume_number
 
+        if options.xmlOutput:
+            folder = Path(options.xmlOutput).resolve()
+            folder.mkdir(parents=True, exist_ok=True)
+            options.output = XMLOutputFormattingWrapper(
+                options.output, folder=folder)
+
         self.options = options
 
         self.features.append(zope.testrunner.selftest.SelfTest(self))
@@ -964,6 +974,29 @@ class TestResult(unittest.TestResult):
         unittest.TestResult.addSkip(self, test, reason)
         self.options.output.test_skipped(test, reason)
 
+    def addSubTest(self, test, subtest, exc_info):
+        if exc_info is None:
+            return
+        stdout, stderr = self._restoreStdStreams()
+        outp = self.options.output
+        report = (outp.test_failure
+                  if issubclass(exc_info[0], test.failureException)
+                  else outp.test_error)
+        report(subtest, time.time() - self._start_time,
+               exc_info, stdout=stdout, stderr=stderr)
+
+        unittest.TestResult.addSubTest(self, test, subtest, exc_info)
+
+        if self.options.post_mortem:
+            if self.options.resume_layer:
+                self.options.output.error_with_banner(
+                    "Can't post-mortem debug when running a layer as"
+                    " a subprocess!")
+            else:
+                zope.testrunner.debug.post_mortem(exc_info)
+        elif self.options.stop_on_error:
+            self.stop()
+
     def addError(self, test, exc_info):
         stdout, stderr = self._restoreStdStreams()
         self.options.output.test_error(test, time.time() - self._start_time,
